/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.render;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collection;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.ValueRange;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.ViewType;
import org.geotools.factory.CommonFactoryFinder;
import org.geotools.filter.expression.ExpressionAbstract;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.geotools.renderer.lite.gridcoverage2d.GridCoverageRenderer;
import org.geotools.styling.*;
import org.opengis.filter.expression.ExpressionVisitor;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.operation.TransformException;
import org.openide.util.Exceptions;

/**
 * GridCoverage2DRenderer
 *
 *
 * @author Timon Veenstra
 */
public class GridCoverage2DRenderer implements RasterRenderer<GridCoverage2D> {

    /**
     * this gets rid of exception for not using native acceleration
     */
    static {
        System.setProperty("com.sun.media.jai.disableMediaLib", "true");
    }
    private StyleFactory sf = CommonFactoryFinder.getStyleFactory(null);

    @Override
    public void paint(GridCoverage2D raster, Rectangle viewport, Graphics2D graphics, Palette palette, GeoTranslator translator) {
        // skip painting is raster is empty
        if (raster == null) {
            return;
        }

        try {
            double minx = raster.getEnvelope2D().getMinX();
            double maxx = raster.getEnvelope2D().getMaxX();
            double miny = raster.getEnvelope2D().getMinY();
            double maxy = raster.getEnvelope2D().getMaxY();
            GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory(null);
            
            Coordinate[] coordinates = new Coordinate[]{new Coordinate(minx, miny), new Coordinate(maxx, miny), new Coordinate(maxx, maxy), new Coordinate(minx, miny)};
            
            CoordinateArraySequence sequence = new CoordinateArraySequence(coordinates);
            LinearRing linearRing = new LinearRing(sequence, geometryFactory);
            Polygon polygon = new Polygon(linearRing, new LinearRing[0], geometryFactory);
            Collection coll = new ArrayList();
            coll.add(polygon);
            Geometry geometry = geometryFactory.buildGeometry(new ArrayList(coll));
            
            final Point2D upLeft = translator.geoToPixel(viewport, geometry , new Coordinate(minx, miny));
            final Point2D downRight = translator.geoToPixel(viewport, geometry, new Coordinate(maxx, maxy));

            Rectangle bounds = new Rectangle(0, 0, -1, -1);

            int x = (int) (upLeft.getX() - viewport.getX());
            int y = (int) (upLeft.getY() - viewport.getY());

            bounds.setLocation(x, y);

            x = (int) (upLeft.getX() - viewport.getX());
            y = (int) (downRight.getY() - viewport.getY());

            bounds.add(x, y);

            x = (int) (downRight.getX() - viewport.getX());
            y = (int) (downRight.getY() - viewport.getY());

            bounds.add(x, y);

            x = (int) (downRight.getX() - viewport.getX());
            y = (int) (upLeft.getY() - viewport.getY());

            bounds.add(x, y);


            Envelope rasterEnvelope = new Envelope(minx, maxx, miny, maxy);
            GridCoverageRenderer renderer = new GridCoverageRenderer(DefaultGeographicCRS.WGS84, rasterEnvelope, bounds, (AffineTransform) null);
            RasterSymbolizer symbolizer = sf.getDefaultRasterSymbolizer();
            symbolizer.setOpacity(new OpacityExp(0.9f));

            if (palette instanceof ValueRange) {
                ValueRange range = (ValueRange)palette;
                Number min = range.getMinValue();
                Number max = range.getMaxValue();
                double step = (max.doubleValue()-min.doubleValue())/range.getSteps();
                for (int s=0;s<range.getSteps();s++){
                    Integer value = new Double(min.doubleValue()+(s*step)).intValue();
                    Color c = palette.getColorForValue(value);
                    symbolizer.getColorMap().addColorMapEntry(getEntry(c,value.intValue()));
                }

            } else {
                //
                //fallback
                //
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF000000, true), 1));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF8F050C, true), 100));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFFDB0F01, true), 200));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFFE65E00, true), 300));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFFFC9F00, true), 400));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFFFAD007, true), 500));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFFDCF40A, true), 600));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF92FC00, true), 700));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF41C405, true), 800));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF048608, true), 900));
                symbolizer.getColorMap().addColorMapEntry(getEntry(new Color(0xFF044606, true), 1000));
            }
            symbolizer.getColorMap().setType(ColorMap.TYPE_RAMP);

            renderer.paint(graphics, raster.view(ViewType.GEOPHYSICS), symbolizer);
        } catch (FactoryException ex) {
            Exceptions.printStackTrace(ex);
        } catch (TransformException ex) {
            Exceptions.printStackTrace(ex);
        } catch (NoninvertibleTransformException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    private ColorMapEntry getEntry(Color color, int value) {
        ColorMapEntry entry = new ColorMapEntryImpl();
        entry.setColor(new ColorExp(color));
        entry.setQuantity(new ValueExp(value));
        return entry;
    }

    private class ColorExp extends ExpressionAbstract {

        private final Color color;

        public ColorExp(Color color) {
            this.color = color;
        }

        @Override
        public Object evaluate(Object object) {
            return this.color;
        }

        @Override
        public Object accept(ExpressionVisitor visitor, Object extraData) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
    }

    private class ValueExp extends ExpressionAbstract {

        private final int value;

        public ValueExp(int value) {
            this.value = value;
        }

        @Override
        public Object evaluate(Object object) {
            return Integer.valueOf(this.value);
        }

        @Override
        public Object accept(ExpressionVisitor visitor, Object extraData) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
    }

    private class OpacityExp extends ExpressionAbstract {

        private final double opacity;

        public OpacityExp(double opacity) {
            assert opacity <= 1.0 && opacity > 0.0;
            this.opacity = opacity;
        }

        @Override
        public Object evaluate(Object object) {
            return new Double(this.opacity);
        }

        @Override
        public Object accept(ExpressionVisitor visitor, Object extraData) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
    }
 }
